{+--------------------------------------------------------------------------+
 | Created:     1.98
 | Author:      Martin Waldenburg
 | Copyright    1997, all rights reserved.
 | Description: Demo for TmEPTokenList, it retrives all classes in an unit.
 | Version:     1.0
 | Status:      FreeWare
 | DISCLAIMER:  This is provided as is, expressly without a warranty of any kind.
 |              You use it at your own risc.
 |
 | Changed      03/1999
 | By           Kevin S. Gallagher
 |              Removed Martin's timer component, removed code for manual
 |              selection of file to read. Moved FFileName from private to
 |              Public.
 +--------------------------------------------------------------------------+}

unit BrowserUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, mwEPTokenList, ComCtrls, ExtCtrls;

type

  TfrmClassBrowser = class(TForm)
    Panel1: TPanel;
    cmdExit: TButton;
    TreeView1: TTreeView;
    procedure TreeView1Deletion(Sender: TObject; Node: TTreeNode);
    procedure FormDestroy(Sender: TObject);
    procedure TreeView1Expanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure cmdExitClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    Parser: TmEPTokenList;
    Stream: TMemoryStream;
    { KSG changed }
    //    FFileName: String;
  public
    { Public-Deklarationen }
    { KSG addition }
    constructor Create(FFileName: String; Owner: TComponent); reintroduce ;
  end;

var
  frmClassBrowser: TfrmClassBrowser;
  LastPos: LongInt;
  Info: TmLPasInfo;

implementation

uses ReadDpr, Global;

{$R *.DFM}

procedure TfrmClassBrowser.TreeView1Deletion(Sender: TObject; Node: TTreeNode);
begin
  if Node.Data <> nil then
  begin
    TmLPasInfo(Node.Data).Free;
    Node.Data:= nil;
  end;
end;

constructor TfrmClassBrowser.Create(FFileName: String; Owner: TComponent);
var
  Data: String;
  ClassKind: TmInfoKind;
  InterfaceKind: TmInfoKind;
  aNode, ClForwardNode, ClassesNode, ClReferenceNode, ExceptionsNode,
  IntForwardNode, InterfacesNode: TTreeNode;
  I: Integer;
begin
  inherited Create(Owner) ;

  Stream:= TMemoryStream.Create;
  Parser:= TmEPTokenList.Create;
  TreeView1.Items.Clear;
  TreeView1.Items.BeginUpdate;
  { KSG modification }
  //if OpenDialog1.Execute then
  //begin
    aNode:= nil;
    Stream.Clear;
    Stream.LoadFromFile(FFileName);
    Parser.SetOrigin(Stream.Memory, Stream.Size);
    ExceptionsNode:= TreeView1.Items.AddChild(nil, 'EXCEPTIONS');
    ClForwardNode:= TreeView1.Items.AddChild(nil, 'CLASS FORWARD');
    ClReferenceNode:= TreeView1.Items.AddChild(nil, 'CLASS REFERENCES');
    ClassesNode:= TreeView1.Items.AddChild(nil, 'CLASSES');
    for I:= 0 to Parser.Searcher.ClassList.Count -1 do
    begin
      Parser.RunIndex:= Parser.Searcher.ClassList[I];
      ClassKind:= Parser.GetClassKind;
      Data:= Parser.Info.Data;
      Case ClassKind of
        ikClass:
          begin
            Case Data[1] <> 'E' of
              True: aNode:= TreeView1.Items.AddChild(ClassesNode, Data);
              False: aNode:= TreeView1.Items.AddChild(ExceptionsNode, Data);
            end;
            aNode.HasChildren:= True;
            Info:= TmLPasInfo.Create;
            Info.ID:= Parser.Info.ID;
            Info.Data:= Parser.Info.Data;
            Info.StartIndex:= Parser.Info.StartIndex;
            Info.AI:= TmAdditionalInfo.Create;
            Info.AI.Ready:= False;
            aNode.Data:= Info;
          end;

        ikClEmpty:
          begin
            Case Data[1] <> 'E' of
              True: TreeView1.Items.AddChild(ClassesNode, Data);
              False: TreeView1.Items.AddChild(ExceptionsNode, Data);
            end;
          end;

        ikClForward:
          TreeView1.Items.AddChild(ClForwardNode, Data);

        ikClReference:
          TreeView1.Items.AddChild(ClReferenceNode, Data);
      end;
    end;
    Parser.Searcher.FillInterfaceList;
    IntForwardNode:= TreeView1.Items.AddChild(nil, 'INTERFACE FORWARD');
    InterfacesNode:= TreeView1.Items.AddChild(nil, 'INTERFACES');
    for I:= 0 to Parser.Searcher.InterfaceList.Count -1 do
    begin
      Parser.RunIndex:= Parser.Searcher.InterfaceList[I];
      InterfaceKind:= Parser.GetInterfaceKind;
      Data:= Parser.Info.Data;
      Case InterfaceKind of
        ikInterface:
          begin
            aNode:= TreeView1.Items.AddChild(InterfacesNode, Data);
            aNode.HasChildren:= True;
            Info:= TmLPasInfo.Create;
            Info.ID:= Parser.Info.ID;
            Info.Data:= Parser.Info.Data;
            Info.StartIndex:= Parser.Info.StartIndex;
            Info.AI:= TmAdditionalInfo.Create;
            Info.AI.Ready:= False;
            aNode.Data:= Info;
          end;

        ikIntEmpty:
          begin
            TreeView1.Items.AddChild(InterfacesNode, Data);
          end;

        ikIntForward:
          TreeView1.Items.AddChild(IntForwardNode, Data);
      end;
    end;
  //end;
  TreeView1.Items.EndUpdate;
end;

procedure TfrmClassBrowser.FormDestroy(Sender: TObject);
begin
  Parser.Free;
  Stream.Free;
end;

procedure TfrmClassBrowser.TreeView1Expanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
begin
  if (Node.Data <> nil) then
  if not TmLPasInfo(Node.Data).AI.Ready then
  begin
    Parser.RunIndex:= TmLPasInfo(Node.Data).StartIndex;
    if TmLPasInfo(Node.Data).ID = ikClass then
    Parser.GetClassKind else
    if TmLPasInfo(Node.Data).ID = ikInterface then
    Parser.GetInterfaceKind;
    repeat
      Parser.NextMember;
      TreeView1.Items.AddChild(Node, Parser.Info.Data);
    until (Parser.Info.ID = ikObEnd) or (Parser.Info.ID =ikError);
    TmLPasInfo(Node.Data).AI.Ready:= True;
  end;
end;

procedure TfrmClassBrowser.cmdExitClick(Sender: TObject);
begin
  Close ;
end;

procedure TfrmClassBrowser.FormResize(Sender: TObject);
begin
  CenterChild(Panel1) ;
end;

end.
