unit Global;
{*********************************************************************}
{ Description:  Project Manager                                       }
{  Programmer:  Kevin S Gallagher     gallaghe@teleport.com           }
{    Requires:  TBrkApart component (FreeWare)                        }
{    Versions:  4 and 5 of Delphi                                     }
{     SeeAlso:  DPRegstr unit                                         }
{*********************************************************************}
interface

uses
  Windows,Dialogs,SysUtils,Forms,Messages,Classes,ShellAPI,FileCtrl,
  Graphics, extctrls ;
  
const
  MainDbf                 = 'Projects.db' ;
  ADD_NEW_PROJECTS        =  1 ;
  EDIT_CURRENT_PROJECT    =  2 ;
  D16                     = '\Delphi.exe' ;
  ABORT_ADDING_RECORD     = -1;
  DEFAULT_PROJECT_FILTER  =  1 ;
  DPR_FILTER              =  1 ;
  VBP_FILTER              =  3 ;
  LANGUAGE_ORDER          =  1 ;
  NO_ORDER                =  2 ;
  Win16Projects           = '16' ;
  PrjFlt                  = 'Delphi project (*.dpr)|*.dpr|' +
                            'Delphi Group (*.bpg)|*.bpg|' +
                            'Visual Basic project (*.VBP)|*.vbp';

procedure RunWinApp(const S: String);
procedure ExploreDirs(const S: String);

function  Question(const S: String):Boolean;
procedure Information(const S: String);
procedure InformationPos(const S: String;nRow,nCol:Integer);
procedure Alert(const S: String);
procedure WarnMsg(const S: String);
procedure ErrMsg(const S: String);
function  RetryBox(const S: String):Boolean;
function  StrCrLf(const cMsg:String):String;

function NtoS(iNumber: Integer): String;
function  Len(S:String):LongInt;

function AppPath:String;
function IsDelphiRunning : boolean;
procedure CenterChild(Panel : TPanel) ;

implementation

uses Main, kg_variousDialogs;

function AppPath:String;
begin
  Result := ExtractFilePath(Application.ExeName);
  Result := Copy(Result,1,Length(Result) -1);
end;

procedure RunWinApp(const S: String);
var
  sFile: string;
begin
  sFile := S + #0;

  if ShellExecute(0,nil, @sFile[1],nil,nil,SW_NORMAL) <= 32 then
    ErrMsg('Could not open desired project' + #13 + S);
end;

procedure ExploreDirs(const S: String);
var
  sCmd:String;
  sDir:String;
begin
  sCmd := 'explore' + #0;
  sDir := ExtractFilePath(S);

  if not DirectoryExists(sDir) then
    exit;

  sDir := sDir + #0;
  ShellExecute(0, @sCmd[1], @sDir[1],nil,nil,SW_SHOWNORMAL);
end;

function  Len(S:String):LongInt;
begin
  Result := Length(S);
end;

function NtoS(iNumber: Integer) : String;
begin
  Result := IntToStr(iNumber);
end;


function StrCrLf(const cMsg:String):String;
var
  nPos:Integer;
  S,cTmp:String;
begin
  S := cMsg;
  if Pos(';',S) = 0 then
  begin
    Result := cMsg;
    exit;
  end;

  repeat
    nPos := Pos(';',S);
    if nPos > 0 then
    begin
      cTmp := cTmp + Copy(S,1,nPos -1) + #13;
      S    := Copy(S,nPos +1,Length(S));
    end;
  until nPos = 0;

  if Length(S) > 0 then
  begin
    Result := cTmp + S + #13;
    S := Result;
  end;
end;

procedure Alert(const S: String);
begin
  MessageDlg(StrCrLf(S),mtWarning,[mbOk],0);
end;

procedure WarnMsg(const S: String);
begin
  MessageDlgCtr(S,mtWarning,[mbOk]) ;
end;

procedure Information(const S: String);
begin
  MessageDlg(StrCrLf(S),mtInformation,[mbOk],0);
end;

procedure InformationPos(const S: String;nRow,nCol:Integer);
begin
  MessageDlgPos(StrCrLf(S),mtInformation,[mbOk],0,nCol,nRow);
end;

procedure ErrMsg(const S: String);
begin
  MessageDlg(StrCrLf(S),mtError,[mbOk],0);
end;

function Question(const S: String):Boolean;
begin
  case MessageDlg(StrCrLf(S),mtConfirmation,[mbYes,mbNo],0) of
     IDYES    : Result := True;
     IDNO     : Result := False;
  else          Result := False;
  end;
end;

function RetryBox(const S: String):Boolean;
begin
  case MessageDlg(StrCrLf(S),mtConfirmation,[mbRetry,mbNo],0) of
     IDRETRY  : Result := True;
     IDNO     : Result := False;
  else          Result := False;
  end;
end;

function IsDelphiRunning : boolean;
begin
  if((FindWindow('TApplication',nil) = 0) and
     (FindWindow('TApplication',nil) = 0) ) or
     (FindWindow('TPropertyInspector',nil) = 0) or
     (FindWindow('TAppBuilder',nil) = 0) then result:= false
  else result:= true;
end;

procedure CenterChild(Panel : TPanel) ;
begin
  Panel.Controls[0].Left := (Panel.Width - Panel.Controls[0].Width) div 2 ;
  Panel.Controls[0].Top := (Panel.Height - Panel.Controls[0].Height) div 2 ;
end ;

end.
