unit InSightCode;
{*********************************************************************}
{ The constant KG_DUMMYDIR is used in "Save File" event, make sure to }
{ read my comments before undefining it.                              }
{*********************************************************************}

{$DEFINE KG_DUMMYDIR}

{$I DFS.INC}  { Standard defines for all Delphi Free Stuff components }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Grids, FileCtrl;

type
  TfrmInSight = class(TForm)
    pnlMenu: TPanel;
    cmdSaveChanges: TButton;
    cmdExit: TButton;
    StatusBar1: TStatusBar;
    pnlCode: TPanel;
    lblShortCut: TLabel;
    lblDescription: TLabel;
    txtShortCut: TEdit;
    txtDesc: TEdit;
    TheCode: TMemo;
    sGrid: TStringGrid;
    Bevel1: TBevel;
    cmdApplyChanges: TButton;
    cmdAddNewShortCut: TButton;
    cmdDeleteShortCut: TButton;
    procedure cmdExitClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure sGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure sGridKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure sGridClick(Sender: TObject);
    procedure sGridEnter(Sender: TObject);
    procedure sGridDblClick(Sender: TObject);
    procedure cmdSaveChangesClick(Sender: TObject);
    procedure cmdApplyChangesClick(Sender: TObject);
    procedure cmdAddNewShortCutClick(Sender: TObject);
    procedure cmdDeleteShortCutClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    function Question(S:String):boolean;
    function GetShortCut:String;
    function GetDesc:String;
    function GetMemoCode:String;
    procedure ApplyNewValues;
    procedure RefreshItems;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmInSight: TfrmInSight;
  aLines:TStringList;

implementation

uses DPRegstr, Main;

{$R *.DFM}

procedure TfrmInSight.cmdExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmInSight.FormActivate(Sender: TObject);
var
  i,x:Integer;
begin

  sGrid.Cells[0,0] := 'Name/ShortCut';
  sGrid.Cells[1,0] := 'Description';

  try
    ReadCodeInSight;

    x := 1;

    for i := 0 to MAX_CODE -1 do
      if length(aString[i,0]) > 0 then
      begin
        Inc(x);
        sGrid.Cells[0,i +1] := aString[i,0];
        sGrid.Cells[1,i +1] := aString[i,1];
        aLines.Add(aString[i,2]);
        { No need to keep element values anymore }
        aString[i,0] := '';
        aString[i,1] := '';
        aString[i,2] := '';
      end;
    sGrid.RowCount := x;

    RefreshItems;
  except  // need to finish up
  end;
end;

procedure TfrmInSight.sGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  RefreshItems;
end;

procedure TfrmInSight.sGridKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  RefreshItems;
end;

function TfrmInSight.GetMemoCode:String;
begin
  Result := aLines.Strings[sGrid.Row -1]
end;

function TfrmInSight.GetShortCut:String;
begin
  Result := sGrid.Rows[sGrid.Row].Strings[0];
end;

procedure TfrmInSight.ApplyNewValues;
var
  S:String;
  i:Integer;
begin
  S := '';
  sGrid.Rows[sGrid.Row].Strings[0] := txtShortCut.text;
  sGrid.Rows[sGrid.Row].Strings[1] := txtDesc.text;
  for i := 0 to TheCode.Lines.Count -1 do
    if i = TheCode.Lines.Count -1 then
      S := S + TheCode.Lines[i]
    else
      S := S + TheCode.Lines[i] +#13#10;

  aLines.Strings[sGrid.Row -1] := S;
end;

function TfrmInSight.GetDesc:String;
begin
  Result := sGrid.Rows[sGrid.Row].Strings[1];
end;

procedure TfrmInSight.RefreshItems;
begin
  try
    TheCode.Lines.Clear;
    TheCode.Lines.Add(GetMemoCode);
    TheCode.SelStart := 0;
    TheCode.SelLength := 0;

    txtShortCut.text := GetShortCut;
    txtDesc.text := GetDesc;
  except
  end;
end;

procedure TfrmInSight.sGridClick(Sender: TObject);
begin
  RefreshItems;
end;

procedure TfrmInSight.sGridEnter(Sender: TObject);
begin
  RefreshItems;
end;

procedure TfrmInSight.sGridDblClick(Sender: TObject);
begin
  RefreshItems;
end;

{*******************************************************************
  Purpose:
  To save changes back to disk. I made a decision not to save
  any changes back to the original file, but instead write the
  changes to the project directory with a filename of [Delphi32.ksg]
  You must alter the code to write back to [Delphi32.dci], the file
  which Delphi reads from. This insures that you can not blame me
  for anything bad occuring!
*******************************************************************}
procedure TfrmInSight.cmdSaveChangesClick(Sender: TObject);
var
  F:Textfile;
  cDir:String;
  cFile:String;
  x:Integer;
  iLen:Integer;
begin

  {$IFDEF KG_DUMMYDIR}
    cFile :=  'Delphi32.ksg';
    cDir := ExtractFilePath(Application.ExeName);
  {$ELSE}
    cFile :=  'Delphi32.dci';
    cDir := WhereIsDelphi + #92  + 'Bin\';
  {$ENDIF}

  if not DirectoryExists(cDir) then
    ShowMessage('Could not locate ' + cDir);

  try
    AssignFile(F,cDir + #92 + cFile);
    ReWrite(F);
    {$IFDEF KG_DUMMYDIR}
    WriteLn(F,'Before undefining KG_DUMMYDIR please inspect all lines');
    WriteLn(F,'to insure they confirm with structure Delphi uses.');
    WriteLn(F,'');
    {$ENDIF}
    for x := 1 to sGrid.RowCount -1 do
    begin
      Writeln(F,
        '[' + trim(sGrid.Rows[x].Strings[0]) + #32 + '| ' +
              trim(sGrid.Rows[x].Strings[1]) +       ']'
      );

      iLen := Length(aLines.Strings[x -1]);

      if iLen >0 then
        Writeln(F,aLines.Strings[x -1]);

      WriteLn(F,#32);
    end;
  finally
    Closefile(F);
  end;
end;

procedure TfrmInSight.cmdApplyChangesClick(Sender: TObject);
begin
  ApplyNewValues;
end;

procedure TfrmInSight.cmdAddNewShortCutClick(Sender: TObject);
begin
  if not Question('Add new item?') then
    exit;

  with sGrid do
  begin
    Row := RowCount -1;
    RowCount := RowCount +1;
    Row := RowCount -1;
    Rows[Row].Strings[0] := 'Enter a shortcut';
    Rows[Row].Strings[1] := 'Enter a Description';
    aLines.Add('Enter your code here!!!');
  end;
  
  RefreshItems;
end;

procedure TfrmInSight.cmdDeleteShortCutClick(Sender: TObject);
var
  n:word;
begin
  with sGrid do begin
    if RowCount -1 = FixedRows then
      exit;

  if not Question('Really delete this item') then
    exit;

    aLines.Delete(sGrid.Row -1);

    for n := Row to RowCount -2 do
      Rows[n] := Rows[n +1];

    RowCount := RowCount -1;
    Rows[RowCount].Clear;
    SetFocus;
  end;
  RefreshItems;
end;

function TfrmInSight.Question(S:String):boolean;
begin
  Result := MessageDlg(S,mtConfirmation,[mbYes,mbNo],0) = mrYes;
end;

procedure TfrmInSight.FormCreate(Sender: TObject);
begin

  {$IFDEF DFS_DELPHI_4}
  Constraints.MaxHeight := 447 ;
  Constraints.MaxWidth := 400 ;
  Constraints.MinHeight := 447 ;
  Constraints.MinWidth := 400 ;
  {$ENDIF}

  Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) -(Width div 2) ;
  Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) -(Height div 2) ;
  
  aLines:= TStringList.Create;
  aLines.Duplicates := dupIgnore;
end;

procedure TfrmInSight.FormDestroy(Sender: TObject);
begin
  aLines.Destroy;
end;
end.
