unit ReadPackage;

{$I DFS.INC}
{***********************************************************************}
{ Description:  Project Manager                                         }
{  Programmer:  Kevin S Gallagher     gallaghe@teleport.com             }
{    Requires:                                                          }
{    Versions:  Tested with 3 and 4                                     }
{    Comments:  This module was originally created for assisting me in  }
{               transitioning from one version of Delphi to another.    }
{***********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TfrmReadDpkFiles = class(TForm)
    lbSearchPaths: TListBox;
    cmdExit: TButton;
    Label1: TLabel;
    lbDpkFiles: TListBox;
    Label2: TLabel;
    lbUnits: TListBox;
    Label3: TLabel;
    StatusBar1: TStatusBar;
    Memo1: TMemo;
    Bevel1: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure cmdExitClick(Sender: TObject);
    procedure lbSearchPathsClick(Sender: TObject);
    procedure lbDpkFilesClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmReadDpkFiles: TfrmReadDpkFiles;

implementation

uses DPRegstr;

{$R *.DFM}

procedure TfrmReadDpkFiles.FormCreate(Sender: TObject);
var
  cTmp :String ;
  cItm :String ;
  iPos :Integer ;
begin

  {$IFDEF DFS_COMPILER_4_UP}
  cTmp := StringReplace(GetSearchPath,'$(DELPHI)',WhereIsDelphi,[rfReplaceAll]) ;
  {$ELSE}
  cTmp := GetSearchPath ;
  {$ENDIF}
  
  repeat
    iPos := Pos(';',cTmp) ;
    cItm := Copy(cTmp,1,iPos -1) ;
    if length(cItm) > 0 then
      lbSearchPaths.Items.Add(cItm) ;
    Delete(cTmp,1,iPos) ;
    Application.ProcessMessages ;
  until Pos(';',cTmp) = 0 ;
  if lbSearchPaths.Items.Count > -1 then
    lbSearchPaths.ItemIndex := 0 ;
end;

procedure TfrmReadDpkFiles.cmdExitClick(Sender: TObject);
begin
  Close ;
end;

procedure TfrmReadDpkFiles.lbSearchPathsClick(Sender: TObject);
var
  Rec   : TSearchRec;
  Res   : Integer ;
  cPath : String ;
begin
  if lbSearchPaths.Items.Count = -1 then    exit ;

  lbDpkFiles.Clear ;
  lbUnits.Clear ;

  cPath := lbSearchPaths.Items.Strings[lbSearchPaths.ItemIndex] ;

  FillChar(Rec,SizeOf(TSearchRec),0) ;
  Res := FindFirst(cPath + '\*.dpk', faAnyFile, Rec) ;
  while Res = 0 do begin
    with Rec do begin
      lbDpkFiles.Items.Add(Rec.Name) ;
    end ;
    Res := FindNext(Rec) ;
  end ;
  FindClose(Rec) ;

  if lbDpkFiles.Items.Count > -1 then
  begin
    lbDpkFiles.ItemIndex := 0 ;
    lbDpkFilesClick(Self) ;
  end ;
end;

procedure TfrmReadDpkFiles.lbDpkFilesClick(Sender: TObject);
  function  LastChrInStr(const S: String): Char ;
  begin
    Result := S[Length(S)] ;
  end;
var
  F    : TextFile ;
  iPos : Integer ;
  cPath: String ;
  cFile: String ;
  cLine: String ;
  cTmp : String ;
begin
  if lbDpkFiles.ItemIndex = -1 then exit ;

  lbUnits.Clear ;

  cPath := lbSearchPaths.Items.Strings[lbSearchPaths.ItemIndex] ;
  cFile := lbDpkFiles.Items.Strings[lbDpkFiles.itemindex] ;
  cLine := '' ;
  
  if not FileExists(cPath + '\' + cFile) then begin
    MessageDlg('Failed to find'#13 + cPath + '\' + cFile,mtError,[mbOk],0) ;
    exit ;
  end ;

  AssignFile(F, cPath + '\' + cFile) ;
  try
    Reset(F) ;
    repeat
      Readln(F, cLine) ;
      Application.ProcessMessages ;
    until UpperCase(cLine) = 'CONTAINS' ;

    while not eof(f) do
    begin
      Readln(F, cLine) ;
      if Trim(cLine) = '' then break ;

      if Pos(' IN ',UpperCase(cLine)) >0 then
      begin
        iPos := Pos(#39,cLine) ;
        cTmp := Copy(cLine,iPos +1,Length(cLine) - iPos) ;
        if not FileExists(cPath + '\' + cTmp) then
          while Length(cTmp) >0 do
          begin
            if not FileExists(cPath + '\' + cTmp) then
              cTmp := Copy(cTmp,1,Length(cTmp) -1)
            else
            begin
              if Pos('.PAS',UpperCase(cTmp)) >0 then
                cTmp := ChangeFileExt(cTmp,'') ;
              lbUnits.Items.Add(cTmp) ;
              break ;
            end ;
            Application.ProcessMessages ;
            if Application.Terminated then
              break ;
          end ;
      end else if (Pos(' IN ',UpperCase(cLine)) = 0) then
      begin
        cLine := Trim(cLine) ;
        lbUnits.Items.Add(Copy(cLine,1,Length(cLine) -1)) ;
      end else if cLine[Length(cLine)] = ';' then
      begin
        cLine := Trim(cLine) ;
        lbUnits.Items.Add(Copy(cLine,1,Length(cLine) -1)) ;
      end ;
    end ;
  finally
    CloseFile(F) ;
  end ;
end;

end.
