unit Unit9;
{-----------------------------------------------------------------------------}
{ Description                                                                 }
{   I wanted to know about a value in the Windows registry which contains the }
{   information used in the Delphi "Direct" dialog. So this form was created  }
{   to show just that.                                                        }
{                                                                             }
{ Author                                                                      }
{   Kevin S. Gallagher  gallaghe@teleport.com                                 }
{                                                                             }
{ Comments                                                                    }
{   The code which builds the tree nodes was done with copy/pasting code from }
{   the first add node. Also the method which holds each nodes "data" might   }
{   be done differently by someone else i.e., not store the date along with   }
{   the ULR.                                                                  }
{-----------------------------------------------------------------------------}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus;

type
  TNodeData = class
     sHyperLink : string ;
  end ;

  TfrmDelphiDirect = class(TForm)
    cmdClose: TButton;
    TreeView1: TTreeView;
    PopupMenu1: TPopupMenu;
    popMnuLink: TMenuItem;
    StatusBar1: TStatusBar;
    procedure cmdCloseClick(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TreeView1Click(Sender: TObject);
  private
    { Private declarations }
    procedure BuildDirectTree ;
    procedure DisposeTreeViewDataItems ;
  public
    { Public declarations }
  end;

var
  frmDelphiDirect: TfrmDelphiDirect;

implementation

uses Registry, DPRegstr, ShellApi ;

{$R *.DFM}

{----------------------------------------------------------------------------
  This code breaks up a value taken from the registry for (sort of) mimicing
  the dialog "Delphi Direct" from the Delphi IDE.
 ----------------------------------------------------------------------------}
procedure TfrmDelphiDirect.BuildDirectTree ;
const
  SplitOne = ' - ' ;

  function StripB(cStr:String): String ;
  begin
    Result := cStr ;
    if (cStr[1] = '[') and (cStr[Length(cStr)] = ']') then
      Result := Copy(cStr,2,Length(cStr) - 2) ;
  end ;
  procedure Builder(aNode:TTreeNode; var S: String) ;
  var
    TS: TTreeNode ;
    cTmp: String ;
    cNode: String ;
    iPos: Integer ;
  begin
    with aNode do
    begin
      Data := TNodeData.Create ;
      TNodeData(Data).sHyperLink := '' ;
      while S[1] <> '['  do
      begin
        iPos := Pos(#13#10,S) ;
        cTmp := Copy(S,1,iPos -1) ;
        System.Delete(S,1,(iPos +1)) ;
        iPos := Pos(SplitOne,cTmp) ;
        if iPos >0 then
        begin
          cNode := Copy(cTmp, iPos +3, Length(cTmp) - iPos) ;
          TS := TreeView1.Items.Add(aNode,cNode) ;
          with TS do
          begin
            Data := TNodeData.Create ;
            TNodeData(Data).sHyperLink := Copy(cTmp,1,iPos -1) ;
          end ;
        end ;
      end ;
    end ;
  end ;
var
  cValue: String ;
  cTmp: String ;
  cNode: String ;
  iPos: Integer ;
  TN,TS: TTreeNode ;
begin
  cValue := GetDelphiDirectHeadLines ;

  { SeeAlso: comments in GetDelphiDirectHeadLines }
  if Length(Trim(cValue)) = 0 then  exit ;
  if Pos(#13#10,cValue)   = 0 then  exit ;

  TreeView1.Items.Clear ;
  TreeView1.Items.BeginUpdate ;

  while Pos(#13#10,cValue) <> 0 do
  begin
    iPos := Pos(#13#10,cValue) ;
    cTmp := Copy(cValue,1,iPos -1) ;
    Delete(cValue,1,iPos +1) ;
    if Length(cTmp) > 2 then
    begin
      if cTmp[1] = '['  then
      begin
        if TreeView1.Selected = nil then
        begin
          if TreeView1.Items.Count = 0 then
          begin
            TN := TreeView1.Items.AddFirst(nil,StripB(cTmp)) ;
            Builder(TN, cValue) ;
            TreeView1.Selected := TN ;
          end else
          begin
            TN := TreeView1.Items.AddChild(nil,cTmp) ;
            Builder(TN, cValue) ;
          end ;
        end  else
        begin
          TN := TreeView1.Items.Add(TreeView1.Selected,StripB(cTmp)) ;
          with TN do
          begin
            Data := TNodeData.Create ;
            TNodeData(Data).sHyperLink := '' ;
            while cValue[1] <> '['  do
              begin
                iPos := Pos(#13#10,cValue) ;
                cTmp := Copy(cValue,1,iPos -1) ;
                System.Delete(cValue,1,(iPos +1)) ;
                iPos := Pos(SplitOne,cTmp) ;
                if iPos >0 then
                begin
                  cNode := Copy(cTmp, iPos +3, Length(cTmp) - iPos) ;
                  TS := TreeView1.Items.AddChild(TN,cNode) ;
                  with TS do
                  begin
                    Data := TNodeData.Create ;
                    TNodeData(Data).sHyperLink := Copy(cTmp,1,iPos -1) ;
                  end ;
                end ;
              end ;
          end ;
        end ;
      end ;
    end ;
  end ;
  TreeView1.Items.EndUpdate ;
  ActiveControl := TreeView1 ;
end;

{----------------------------------------------------------------------------
  Used to free up TreeView node data when closing the form.
 ----------------------------------------------------------------------------}
procedure TfrmDelphiDirect.DisposeTreeViewDataItems ;
var
  iCount: Integer ;
  i: Integer ;
  TheNode: TTreeNode ;
begin
  iCount := TreeView1.Items.Count ;

  for i := 1 to iCount do
  begin
    if i = 1 then
      TheNode := TreeView1.Items[0]
    else
      TheNode := TheNode.GetNext ;
      
    with TheNode do
      if Data <> nil then
        TNodeData(Data).Free ;
  end;
end;

procedure TfrmDelphiDirect.PopupMenu1Popup(Sender: TObject);
var
  S : String ;
  iPos : Integer ;
  TheSelectedNode : TTreeNode ;
begin
  popMnuLink.enabled := False ;
  popMnuLink.Caption := '<None>' ;

  if TreeView1.Selected = nil
    then Exit;

  TheSelectedNode := TreeView1.Selected ;
  S := TNodeData(TheSelectedNode.Data).sHyperLink ;

  if Length(S) =0 then
    exit ;

  iPos := Pos('=',S) ;
  S := Copy(S,1, iPos -1) ;

  popMnuLink.Caption := S ;
  popMnuLink.enabled := True ;

end;

procedure TfrmDelphiDirect.FormResize(Sender: TObject);
var
  cw,ch: Integer ;
begin
  cw := ClientWidth ;
  ch := ClientHeight ;

  cmdClose.Left := (cw - cmdClose.Width) - 2 ;
  cmdClose.Top := (ch - StatusBar1.Height)  - cmdClose.Height - 2 ;

  TreeView1.Width := cw - 2 ;
  TreeView1.Height := (ch - StatusBar1.Height) - (cmdClose.Height + 10) ;
end;

procedure TfrmDelphiDirect.FormCreate(Sender: TObject);
begin
  Caption := 'Delphi ' + DelphiVersionStr + ' Direct information' ;
  BuildDirectTree ;
end;

{----------------------------------------------------------------------------
  Before closing the form free up Node data (in the OnClose event for this
  form) followed by clearing the nodes. The form visible property is set to
  false so the user does not have to wait for the tree to clear on screen.
 ----------------------------------------------------------------------------}
procedure TfrmDelphiDirect.cmdCloseClick(Sender: TObject);
begin
  Close ;
end;

{----------------------------------------------------------------------------
  Code used to connect up to internet for current node web page address
 ----------------------------------------------------------------------------}
procedure TfrmDelphiDirect.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Self.Visible := False ;
  Application.ProcessMessages ;
  DisposeTreeViewDataItems ;
  TreeView1.Items.Clear ;
  Application.ProcessMessages ;
end;

procedure TfrmDelphiDirect.TreeView1Click(Sender: TObject);
var
  S: String ;
begin
  if TreeView1.Selected = nil then  exit ;

  S := TNodeData(TreeView1.Selected.Data).sHyperLink ;
  if Length(S) >0 then
  begin
    S := S + #0 ;
    if ShellExecute(0,nil, @S[1],nil,nil,SW_NORMAL) <= 32 then
      MessageDlg('Failed to connect', mtError,[mbOk],0) ;
  end ;

end;

end.
